/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the Clippings extension for Mozilla Firefox.
 *
 * The Initial Developer of the Original Code is 
 * Alex Eng <ateng@users.sourceforge.net>.
 * Portions created by the Initial Developer are Copyright (C) 2005
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * ***** END LICENSE BLOCK ***** */

//
// Integration with host application
//


var gClippingsExec = {
  dataSrcInitialized: false,
  showDialog: true,

  newFromClipboard: function () 
  {
    if (! this.dataSrcInitialized) {
      // The RDF data source has to be initialized if it has not already been
      // done so, otherwise RDF node creation will fail, and the new Clippings
      // entry will never be created.
      // This initialization is done in the code for the Clippings popup menu's
      // `onpopupshowing' event handler.
      this.initClippingsPopup(document.getElementById("ae-clippings-popup-1"));
    }
    var result = gClippings.newFromClipboard(this.showDialog);
    // rv is undefined if clipboard contents are non-textual,
    // -1 if the user cancelled from the Add New dialog box,
    // or contains the URI of the new RDF node if entry successfully created.
    if (! result) {
      var strBundle = document.getElementById("ae-clippings-strings");
      alert(strBundle.getString('errorNoClipboardContent'));
    }
    else if (result != -1) {
      gClippings.flushDataSrc();
    }
  },

  newFromTextbox: function ()
  {
    if (! this.dataSrcInitialized) {
      this.initClippingsPopup(document.getElementById("ae-clippings-popup-1"));
    }

    var result = false;
    if (gContextMenu.onTextInput) {
      // Get the node where the context menu is invoked from.
      var textbox = document.popupNode;
      if (textbox instanceof HTMLInputElement || textbox instanceof HTMLTextAreaElement) {
	var text;
	if (textbox.selectionStart == textbox.selectionEnd) {
	  text = textbox.value;
	}
	else {
	  text = textbox.value.substring(textbox.selectionStart, 
					 textbox.selectionEnd);
	}
	result = gClippings.newClippingFromText(text, this.showDialog);
      }
      else {
	gClippings.log("In gClippingsExec.newFromTextbox(): textbox = " 
		       + textbox);
      }
    }
    // Rich edit field
    else if (gContextMenu.inFrame) {
      var doc = document.popupNode.ownerDocument;
      if (doc.designMode == "on") {
	var plainText = doc.defaultView.getSelection();

	// New (from entire contents)
	if (plainText == "") {
	  goDoCommand('cmd_selectAll');
	  plainText = doc.defaultView.getSelection();
	  var isTextSelected = true;
	}
	result = gClippings.newClippingFromText(plainText, this.showDialog);
	if (isTextSelected) {
	  // TO DO: Deselect text, leaving the cursor where it was.
	}
      }
    }

    if (result && result != -1) {
      gClippings.flushDataSrc();
    }
  },

  newFromSelection: function ()
  {
    if (! this.dataSrcInitialized) {
      this.initClippingsPopup(document.getElementById("ae-clippings-popup-1"));
    }
    var focusedWnd = document.commandDispatcher.focusedWindow;
    var selection = focusedWnd.getSelection();
    if (selection) {
      gClippings.newClippingFromText(selection, this.showDialog);
      gClippings.flushDataSrc();
    }
    else {
      var strBundle = document.getElementById("ae-clippings-strings");
      alert(strBundle.getString('errorNoSelection'));
    }
  },

  openClippingsManager: function () 
  {
    window.open("chrome://clippings/content/clippings.xul", "clippings_wndobj",
		"chrome");
  },

  initClippingsPopup: function (aPopup) 
  {
    var popup = aPopup;
    var rootNode = gClippings.RDFSvc.GetResource("http://clippings.mozdev.org/rdf/user-clippings");

    try {
      // If Clippings data file doesn't exist, it will be created automatically.
      if (! gClippings.dataSrc) {
	gClippings.dataSrc = gClippings.RDFSvc.GetDataSourceBlocking(gClippings.getCurrentProfileURL() + "/" + CLIPDAT_FILE_NAME);
      }
    }
    catch (e) {
      gClippings.log(e);
    }

    popup.database.AddDataSource(gClippings.dataSrc);

    try {
      gClippings.RDFContainer = gClippings.RDFContainerUtils.MakeSeq(gClippings.dataSrc, rootNode);
      gClippings.RDFContainer = gClippings.RDFContainer.QueryInterface(Components.interfaces.nsIRDFContainer);
    }
    catch (e) {
      gClippings.log(e);
    }

    popup.builder.rebuild();
    this.dataSrcInitialized = true;
  },

  insertClippingText: function (aText) 
  {
    gClippings.copyTextToClipboard(aText);

    // Paste clipping.  The following function is defined in
    // "chrome://global/content/globalOverlay.js"
    goDoCommand("cmd_paste");
    // SIDE EFFECT: The clipping text will remain on the system clipboard.
  },


  //
  // Browser window and Clippings menu initialization
  //

  initClippings: function (event)
  {   
    gClippings.purgeTempData();

    var hostAppContextMenu = document.getElementById("contentAreaContextMenu");
    hostAppContextMenu.addEventListener("popupshowing", function (e) {gClippingsExec.initContextMenuItem(e)}, false);
    hostAppContextMenu.addEventListener("popuphiding", function (e) {gClippingsExec.hideContextMenuItem(e)}, false);

    this.showDialog = !(gClippings.getPref("clippings.entries.add_silently") || false);

    var newFromTextbox = document.getElementById("ae-clippings-add");
    var newFromSelect = document.getElementById("ae-clippings-new-from-select");
    var newFromClipbd = document.getElementById("ae-clippings-new-from-clipbd");
    var strBundle = document.getElementById("ae-clippings-strings");
    // Add elipsis (...) to menu commands if the Add dialog isn't suppressed.
    var ellipsis = this.showDialog ? strBundle.getString("ellipsis") : "";
    newFromTextbox.setAttribute("label", strBundle.getString("new") + ellipsis);
    newFromSelect.setAttribute("label", strBundle.getString("newFromSelect") + ellipsis);
    newFromClipbd.setAttribute("label", strBundle.getString("newFromClipbd") + ellipsis);
  },

  initContextMenuItem: function (event)
  {
    // WORKAROUND: Although this event handler is attached to the browser's 
    // context menu, it gets called 2 times: for the browser context menu, and 
    // for the Clippings submenu. We only want the following to execute when
    // the browser's context menu is being shown, not the Clippings submenu.
    if (event.target.id != "contentAreaContextMenu") {
      return;
    }

    var strBundle = document.getElementById("ae-clippings-strings");
    var ellipsis = this.showDialog ? strBundle.getString("ellipsis") : "";
    var clippingsMenu1 = document.getElementById("ae-clippings-menu-1");
    var clippingsMenu2 = document.getElementById("ae-clippings-menu-2");
    var addEntryCmd = document.getElementById("ae_new_clipping_from_textbox");

    if (gContextMenu.onTextInput) {
      var textbox = document.popupNode;
      // Do our own checking to ensure that the node is an HTML text field.
      if (textbox instanceof HTMLInputElement || textbox instanceof HTMLTextAreaElement) {
	if (textbox.selectionStart == textbox.selectionEnd) {
	  // No text selected.  Enable "New..." cmd if text field non-empty
	  addEntryCmd.setAttribute("label", strBundle.getString("new") + ellipsis);
	  addEntryCmd.setAttribute("disabled", textbox.value == "");
	}
	else {
	  // "New From Selection..." command.
	  addEntryCmd.setAttribute("label", strBundle.getString("newFromSelect") + ellipsis);
	  addEntryCmd.setAttribute("disabled", "false");
	}
      }
      else {
	addEntryCmd.setAttribute("disabled", "true");
      }
      clippingsMenu1.hidden = false;
      clippingsMenu2.hidden = true;
    }
    else if (gContextMenu.isTextSelected) {
      var doc = document.popupNode.ownerDocument;
      if (doc.designMode == "on") {
	clippingsMenu1.hidden = false;
	clippingsMenu2.hidden = true;
      } 
      else {
	clippingsMenu1.hidden = true;
	clippingsMenu2.hidden = false;
      }
      addEntryCmd.setAttribute("disabled", "false");
      addEntryCmd.setAttribute("label", strBundle.getString("newFromSelect") 
			       + ellipsis);
    }
    // Rich edit field - if there is any text selected, the preceding
    // "else if ..." statement will be executed.
    else if (gContextMenu.inFrame) {
      var doc = document.popupNode.ownerDocument;
      if (doc.designMode == "on") {
	// Check for empty document
	var range = doc.createRange();
 	range.setStart(doc.body.firstChild, 0);
 	range.setEnd(doc.body.lastChild, 0);
	addEntryCmd.setAttribute("disabled", range == "");

	clippingsMenu1.hidden = false;
	clippingsMenu2.hidden = true;
	addEntryCmd.setAttribute("label", strBundle.getString("new") + ellipsis);
      }
    }
  },

  hideContextMenuItem: function (event)
  {
    var clippingsMenu1 = document.getElementById("ae-clippings-menu-1");
    var clippingsMenu2 = document.getElementById("ae-clippings-menu-2");

    if (event.target.id == "contentAreaContextMenu") {
      clippingsMenu1.hidden = true;
      clippingsMenu2.hidden = true;
    }
  }
};

// Keep in mind Gecko bug 241518 - calling addEventListener with a closure 
// holding a content node leaks the document (fixed in Firefox 1.6a1)
window.addEventListener("load", 
			function (e) { gClippingsExec.initClippings(e); },
			true);



// Drag 'n drop support
var clippingsStatusBarObserver = {
  onDragOver: function (event, flavour, session)
  {
    event.target.setAttribute("ae-dnd", "true");
  },

  onDragExit: function (event, session)
  {
    event.target.removeAttribute("ae-dnd");
  },

  onDrop: function (event, transferData, session)
  {
    if (! gClippingsExec.dataSrcInitialized) {
      // The RDF data source has to be initialized if it has not already been
      // done so, otherwise RDF node creation will fail, and the new Clippings
      // entry will never be created.
      // This initialization is done in the code for the Clippings popup menu's
      // `onpopupshowing' event handler.
      gClippingsExec.initClippingsPopup(document.getElementById("ae-clippings-popup-1"));
    }

    var text = transferData.data;
    if (gClippings.newClippingFromText(text, gClippingsExec.showDialog)) {
      gClippings.flushDataSrc();
    }
  },

  getSupportedFlavours: function ()
  {
    var flavours = new FlavourSet();
    flavours.appendFlavour("text/unicode");
    return flavours;
  }
};


